//
// CustomLogging.m
//
// Created by Wagner Truppel.
// Copyright 2009 Wagner Truppel (jrcapab@gmail.com). All rights reserved.
//
// Redistribution and use in binary and source forms, with or without
// modification, are permitted for any project, commercial or otherwise,
// provided that the following conditions are met:
//
// Redistributions in binary form must display the copyright notice in the
// About view, website, and/or documentation.
//
// Redistributions of source code must retain the copyright notice, this
// list of conditions, and the following disclaimer.
//
// THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THIS SOFTWARE.

// ========================================================================= //
                        #pragma mark static variables
// ========================================================================= //

static NSInteger depth = -1;

// ========================================================================= //
                           #pragma mark functions
// ========================================================================= //

void WTNil(NSString *format, ...)
{}

// ========================================================================= //

void WTLog(NSString *format, ...)
{
    NSString *name;

    if ([NSThread isMainThread])
    { name = @"Main"; }
    else
    { name = [[NSThread currentThread] name]; }

    name = [[NSString alloc] initWithFormat: @"(%@)", name];

    va_list  args;
    va_start(args, format);
    {
        NSString *message = [[NSString alloc]
            initWithFormat: format arguments: args];

        NSString *output = [[NSString alloc]
            initWithFormat: @"%@ %@", name, message];

        puts([output UTF8String]);

        [output release];
        [message release];
    }
    va_end(args);

    [name release];
}

// ========================================================================= //

void WTTab(NSString *format, ...)
{
    if (depth < 0)
    {
        NSString *output = [[NSString alloc]
            initWithFormat: @"Unmatched call stack debug depth: %i", depth];

        puts([output UTF8String]);
        [output release];

        depth = 0;
    }

    NSMutableString* spaces = [[NSMutableString alloc] initWithCapacity: depth];
    for (NSUInteger i = 0; i < depth; ++i)
    { [spaces appendString: @"  "]; }

    NSString *name;

    if ([NSThread isMainThread])
    { name = @"Main"; }
    else
    { name = [[NSThread currentThread] name]; }

    name = [[NSString alloc] initWithFormat: @"(%@)", name];

    va_list  args;
    va_start(args, format);
    {
        NSString *message = [[NSString alloc]
            initWithFormat: format arguments: args];

        NSString *output = [[NSString alloc]
            initWithFormat: @"%@%@ %@", spaces, name, message];

        puts([output UTF8String]);

        [output release];
        [message release];
    }
    va_end(args);

    [name release];
    [spaces release];
}

// ========================================================================= //

void LogEnter(void)
{
    depth += 1;
}

// ========================================================================= //

void LogExit(void)
{
    depth -= 1;
}

// ========================================================================= //
